/**
 *  Copyright 2011 Wordnik, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.wordnik.swagger.codegen.config;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Maintains the overriding rules that we should use while generating the code.
 *
 * Example; If we need to ignore any REST methods or if we need special service extention classes they can be
 * supplied through this configuration
 *
 * User: ramesh
 * Date: 4/26/11
 * Time: 8:01 AM
 */
public interface RulesProvider {

    /**
     * Checks if the method needs to be ignored when generating libraries.
     *
     * This is used if clients want to override the implementation of the method and not use methods generated by the code gen.
     *
     * @param serviceName
     * @param methodName
     * @return
     */
    public boolean isMethodIgnored(String serviceName, String methodName);

    /**
     * Checks if model needs to be ignored in code generation.
     *
     * @param modelName
     * @return
     */
    public boolean isModelIgnored(String modelName);
}
